return {

    { -- display undo tree for a file
        "https://github.com/mbbill/undotree",
        cmd = "UndotreeToggle",
        keys = {
            { "<leader>ut", "<cmd>UndotreeToggle<cr>",    desc = "undo tree" },
        },
    },

    { -- markview.nvim
        "https://github.com/OXY2DEV/markview.nvim",
        ft = 'markdown',
        keys = {
            { -- toggle markview rendering
                "<localleader>t",
                "<cmd>Markview toggle<cr>",
                desc = "Toggle markdown rendering",
                ft = "markdown"
            },
        },
        opts = {
            modes = { "n", "i", "no", "c" },
            hybrid_modes = { "i" },
            callbacks = {
                on_enable = function (_, win)
                    vim.wo[win].conceallevel = 2;
                    vim.wo[win].concealcursor = "nc";
                end
            }
        },
    },
    { -- repl integration
        "https://github.com/Vigemus/iron.nvim",
        keys = {
            { "<leader>rs", "<cmd>IronRepl<cr>", desc = "Show/start REPL" },
            { "<leader>rh", "<cmd>IronHide<cr>", desc = "Hide REPL" },
            { "<leader>rf", "<cmd>IronFocus<cr>", desc = "Focus REPL" },
            { "<leader>rr", "<cmd>IronRestart<cr>", desc = "Restart REPL" },
            { "<leader>rq", function() require("iron").core.close_repl(vim.bo.filetype) end, desc = "Quit REPL" },
        },
        opts = {
            config = {
                -- Whether a repl should be discarded or not
                scratch_repl = true,
                -- Your repl definitions come here
                repl_definition = {
                    sh = {
                        -- Can be a table or a function that
                        -- returns a table (see below)
                        command = {"zsh"}
                    },
                    julia = {
                        command = {"julia", "--project"},
                    },
                },
                repl_open_cmd = "vertical botright 80 split", -- require("iron").view.split.vertical.botright(100),
            },
            -- Iron doesn't set keymaps by default anymore.
            -- You can set them here or manually add keymaps to the functions in iron.core
            keymaps = {
                send_motion = "<leader>ss",
                visual_send = "<leader>ss",
                send_file = "<space>sf",
                send_line = "<space>sl",
                send_until_cursor = "<space>su",
                -- send_mark = "<space>sm",
                mark_motion = "<space>mc",
                mark_visual = "<space>mc",
                remove_mark = "<space>md",
                cr = "<space>s<cr>",
                interrupt = "<space>s<space>",
                clear = "<space>cl",
            },
            -- If the highlight is on, you can change how it looks
            -- For the available options, check nvim_set_hl
            highlight = {
                italic = false,
            },
            ignore_blank_lines = true, -- ignore blank lines when sending visual select lines
        },
        config = function (_, opts)
            local iron = require("iron.core")
            iron.setup(opts)

            IronSendCell = function(next_cell)
                -- source: https://github.com/Vigemus/iron.nvim/issues/236#issuecomment-1317674256
                local iron_core = require('iron.core')
                local marks = require('iron.marks')

                local cell_pattern = "^\\s*##"  -- cell delimiter pattern
                local cell_start = vim.fn.search(cell_pattern, 'bcnW')
                local cell_end = vim.fn.search(cell_pattern, 'nW')

                local lines = vim.api.nvim_buf_get_lines(0, cell_start, cell_end-1, 0)
                -- ignore blank lines
                local b_lines = {}
                for _, line in ipairs(lines) do
                    if line:gsub("^%s*(.-)%s*$", "%1") ~= '' then
                        table.insert(b_lines, line)
                    end
                end
                if #b_lines == 0 then return end

                if cell_start == 0 then
                    cell_start = 1 -- if first cell, then start from first line
                end
                if cell_end == 0 then
                    cell_end = vim.fn.line('$') -- set to last line
                end

                marks.set{ from_line=cell_start, from_col=0, to_line=cell_end, to_col=-1 }
                marks.winrestview()

                iron_core.send(nil, b_lines)
                if next_cell == true then
                    vim.fn.cursor(cell_end+1, 0) -- move to next cell start
                end
            end

            vim.keymap.set('n', '<space>sc', function () IronSendCell(false) end)
            vim.keymap.set('n', '<space>sC', function () IronSendCell(true) end)
            vim.cmd("nmap <leader>sp <leader>ssap")
        end
    },
    { -- neodev
        "https://github.com/folke/lazydev.nvim",
        ft = "lua",
    },
    { -- A better user experience for interacting with and manipulating marks
        "https://github.com/chentoast/marks.nvim",
        event = "BufReadPost",
        config = true,
    },
    {
        "folke/snacks.nvim",
        priority = 1000,
        lazy = false,
        keys = {
            { "<leader>pp", function () require("snacks").picker() end, desc = "pick picker", },
            { "<leader>pg", function () require("snacks").picker.grep() end, desc = "pick string", },
            { "<leader>pf", function () require("snacks").picker.files() end, desc = "pick files", },
            {
                "<leader>gl",
                function()
                    require("snacks").picker.git_log({
                        finder = "git_log",
                        format = "git_log",
                        preview = "git_show",
                        confirm = "git_checkout",
                        layout = "vertical",
                    })
                end,
                desc = "Git Log",
            },
            {
                "<leader>gm",
                function()
                    require("snacks").picker.git_branches({
                        layout = "select",
                    })
                end,
                desc = "pick branches",
            },
            {
                "<leader><space>",
                function()
                    require("snacks").picker.files({
                        finder = "files",
                        format = "file",
                        show_empty = true,
                        supports_live = true,
                        -- In case you want to override the layout for this keymap
                        -- layout = "vscode",
                    })
                end,
                desc = "Find Files",
            },
            -- Navigate my buffers
            {
                "<leader>pb",
                function()
                    require("snacks").picker.buffers({
                        -- I always want my buffers picker to start in normal mode
                        on_show = function()
                            vim.cmd.stopinsert()
                        end,
                        finder = "buffers",
                        format = "buffer",
                        hidden = false,
                        unloaded = true,
                        current = true,
                        sort_lastused = true,
                        win = {
                            input = {
                                keys = {
                                    ["d"] = "bufdelete",
                                },
                            },
                            list = { keys = { ["d"] = "bufdelete" } },
                        },
                        -- In case you want to override the layout for this keymap
                        -- layout = "ivy",
                    })
                end,
                desc = "[P]Snacks picker buffers",
            },
        },
        opts = {
            picker = {
                matcher = {
                    frecency = true,
                },
                win = {
                    input = {
                        keys = {
                            -- to close the picker on ESC instead of going to normal mode,
                            -- add the following keymap to your config
                            ["<Esc>"] = { "close", mode = { "n", "i" } },
                            -- I'm used to scrolling like this in LazyGit
                            ["J"] = { "preview_scroll_down", mode = { "i", "n" } },
                            ["K"] = { "preview_scroll_up", mode = { "i", "n" } },
                            ["H"] = { "preview_scroll_left", mode = { "i", "n" } },
                            ["L"] = { "preview_scroll_right", mode = { "i", "n" } },
                        },
                    },
                },
            },
            image = {
                enabled = true,
                doc = {
                    inline = false,
                    float = true,
                    max_width = 60,
                    max_height = 30,
                },
            },
            scroll = { enabled = false },
            bigfile = { enabled = true },
            notifier = { enabled = true },
            quickfile = { enabled = true },
            words = { enabled = true },
            dashboard = {
                sections = {
                    {
                        section = "terminal",
                        cmd = "chafa ~/i/piksela_arto/docks_1041uuu_Toyoi_Yuuta.gif --format symbols --symbols vhalf --size 60x17 --stretch; sleep .1",
                        height = 17,
                        padding = 1,
                    },
                    {
                        pane = 2,
                        { section = "keys", gap = 1, padding = 1 },
                        { section = "startup" },
                    },
                },
            },
        },
    },

    {
        {
            "mfussenegger/nvim-dap",
            dependencies = {
                "rcarriga/nvim-dap-ui",
                "theHamsta/nvim-dap-virtual-text",
                "nvim-neotest/nvim-nio",
                "kdheepak/nvim-dap-julia",
            },
            config = function()
                local dap = require("dap")
                local ui = require("dapui")

                require("dapui").setup()
                require("nvim-dap-julia").setup()

                vim.keymap.set("n", "<leader>db", dap.toggle_breakpoint)
                vim.keymap.set("n", "<leader>dB", dap.run_to_cursor)

                vim.keymap.set("n", "<F1>", dap.continue)
                vim.keymap.set("n", "<F2>", dap.step_into)
                vim.keymap.set("n", "<F3>", dap.step_over)
                vim.keymap.set("n", "<F4>", dap.step_out)
                vim.keymap.set("n", "<F5>", dap.step_back)
                vim.keymap.set("n", "<F12>", dap.restart)

                dap.listeners.before.attach.dapui_config = function()
                    ui.open()
                end
                dap.listeners.before.launch.dapui_config = function()
                    ui.open()
                end
                dap.listeners.before.event_terminated.dapui_config = function()
                    ui.close()
                end
                dap.listeners.before.event_exited.dapui_config = function()
                    ui.close()
                end
            end,
        },
    },

    {
        "https://github.com/chomosuke/typst-preview.nvim",
        ft = "typst",
        version = "1.*",
        keys = {
            { "<localleader>pd", "<CMD>TypstPreview document<CR>", desc = "open typst preview document", },
            { "<localleader>ps", "<CMD>TypstPreview slide<CR>", desc = "open typst preview slide", },
            { "<localleader>s", "<CMD>TypstPreviewStop<CR>", desc = "stop typst preview", },
            { "<localleader><SPACE>", "<CMD>TypstPreviewSyncCursor<CR>", desc = "sync cursor", },
            { "<localleader>f", "<CMD>TypstPreviewFollowCursorToggle<CR>", desc = "toggle follow cursor", },
        },
        opts = {}, -- lazy.nvim will implicitly calls `setup {}`
    },

    {
        "https://github.com/hat0uma/csvview.nvim",
        ft = {"csv", "tsv"},
        opts = {},
        keys = {
            { "<localleader>e", "<CMD>CsvViewEnable<CR>",   desc = "enable csv preview", },
            { "<localleader>d", "<CMD>CsvViewDisable<CR>",  desc = "disable csv preview", },
            { "<localleader>t", "<CMD>CsvViewToggle<CR>",   desc = "toggle csv preview", },
        },
    },

}
